package com.netpc.recruitment.controllers;

import com.netpc.recruitment.models.contact.Contact;
import com.netpc.recruitment.models.contact.IContactDAO;
import com.netpc.recruitment.models.contact.NoSuchContactException;
import com.netpc.recruitment.models.user.UserAuth;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import java.text.SimpleDateFormat;

/**
 * @author superuser
 *         Created 13-Jan-15
 */
@Controller
@RequestMapping(value = "/contact")
public class ContactController {
    @Autowired
    private IContactDAO contactDAO;

    @Autowired
    private UserAuth userAuth;

    private static final String VIEW_INDEX = "edit";

    @RequestMapping(value = "/edit", method = RequestMethod.GET)
    public String editAction(@RequestParam(required = false) String id, String error, ModelMap model) {
        if (!userAuth.isLogged())
            return "redirect:/";

        model.addAttribute("error", error);
        try {
            Contact user = null;
            if (id != null)
                user = contactDAO.getContact(Integer.valueOf(id));
            else
                user = new Contact();

            model.addAttribute("contact", user);
        } catch (NoSuchContactException e) {
            return "redirect:/";
        } catch (Exception e) {
            e.printStackTrace();
        }

        return VIEW_INDEX;
    }

    @RequestMapping(value = "/edit", method = RequestMethod.POST)
    public String editAction(@RequestParam String id, String firstName, String secondName, String email, String phone, String dob, ModelMap model) {
        if (!userAuth.isLogged())
            return "redirect:/";
        SimpleDateFormat sdf1 = new SimpleDateFormat("dd-MM-yyyy");
        try {
            if (!"0".equals(id)) {
                Contact edit = new Contact(Integer.parseInt(id), firstName, secondName, email, phone, sdf1.parse(dob));
                contactDAO.updateContact(edit);
            } else {
                contactDAO.createContact(new Contact(firstName, secondName, email, phone, sdf1.parse(dob)));
            }
        } catch (Exception e) {
            return "redirect:/contact/edit?id=" + id + "&error=" + e.getMessage();
        }

        return "redirect:/";
    }

    @RequestMapping(value = "/delete", method = RequestMethod.GET)
    public String deleteAction(@RequestParam int id) {
        if (!userAuth.isLogged())
            return "redirect:/";
        try {
            if (!contactDAO.deleteContact(new Contact(id)))
                throw new Exception("Failed to delete user!");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "redirect:/";
    }

}
