package com.netpc.recruitment.models.contact;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.sql.Connection;
import javax.sql.DataSource;

/**
 * @author superuser
 *         Created 12-Jan-15
 */
public class JDBCContactDAO implements IContactDAO {
    private static final String SELECT_ALL = "SELECT * FROM PEOPLE";
    private static final String SELECT_CONTACT = "SELECT * FROM PEOPLE WHERE id = ?";
    private static final String UPDATE = "UPDATE PEOPLE SET first_name = ?, second_name = ?, email = ?, phone = ?, dob = ? WHERE id = ?";
    private static final String CREATE = "INSERT INTO PEOPLE (first_name, second_name, email, phone, dob) VALUES (?, ?, ?, ?, ?)";
    private static final String DELETE = "DELETE FROM PEOPLE WHERE id = ?";

    private DataSource dataSource;

    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public List<Contact> getAllContacts() throws SQLException {
        Connection conn = null;

        try {
            conn = dataSource.getConnection();
            Statement statement = conn.createStatement();
            ResultSet resultSet = statement.executeQuery(SELECT_ALL);
            List<Contact> result = new ArrayList<>();

            while (resultSet.next()) {
                Contact contactBean = new Contact(resultSet.getInt("id"),
                        resultSet.getString("first_name"),
                        resultSet.getString("second_name"),
                        resultSet.getString("email"),
                        resultSet.getString("phone"),
                        resultSet.getDate("dob"));

                result.add(contactBean);
            }

            return result;
        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignored) {}
            }
        }
    }

    @Override
    public Contact getContact(int id) throws NoSuchContactException {
        Connection conn = null;

        try {
            conn = dataSource.getConnection();
            PreparedStatement statement = conn.prepareStatement(SELECT_CONTACT);
            statement.setInt(1, id);
            ResultSet resultSet = statement.executeQuery();

            boolean hasUser = resultSet.next();
            if (!hasUser)
                throw new NoSuchContactException("Contact not found!");

            return new Contact(resultSet.getInt("id"),
                    resultSet.getString("first_name"),
                    resultSet.getString("second_name"),
                    resultSet.getString("email"),
                    resultSet.getString("phone"),
                    resultSet.getDate("dob"));
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignored) {}
            }
        }
    }

    @Override
    public int updateContact(Contact contact) throws SQLException {
        if (contact == null)
            throw new IllegalArgumentException("Object is null");
        else if (contact.getId() == 0)
            throw new IllegalArgumentException("Contact doesn't has id");

        Connection conn = null;

        try {
            conn = dataSource.getConnection();
            PreparedStatement statement = conn.prepareStatement(UPDATE);
            statement.setString(1, contact.getFirstName());
            statement.setString(2, contact.getSecondName());
            statement.setString(3, contact.getEmail());
            statement.setString(4, contact.getPhone());
            statement.setDate(5, new Date(contact.getDob().getTime()));
            statement.setInt(6, contact.getId());

            return statement.executeUpdate();
        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignored) {}
            }
        }
    }

    @Override
    public boolean deleteContact(Contact contact) throws Exception {
        if (contact == null)
            throw new IllegalArgumentException("Object is null");
        else if (contact.getId() == 0)
            throw new IllegalArgumentException("Contact doesn't has id or id is invalid");

        Connection conn = null;
        try {
            conn = dataSource.getConnection();
            PreparedStatement statement = conn.prepareStatement(DELETE);
            statement.setInt(1, contact.getId());

            return statement.executeUpdate() > 0;
        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignored) {}
            }
        }
    }

    @Override
    public void createContact(Contact contact) throws SQLException {
        if (contact == null)
            throw new IllegalArgumentException("Object is null");

        Connection conn = null;
        try {
            conn = dataSource.getConnection();
            PreparedStatement statement = conn.prepareStatement(CREATE);
            statement.setString(1, contact.getFirstName());
            statement.setString(2, contact.getSecondName());
            statement.setString(3, contact.getEmail());
            statement.setString(4, contact.getPhone());
            statement.setDate(5, new Date(contact.getDob().getTime()));

            statement.executeUpdate();
        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignored) {}
            }
        }

    }
}
